"""
Module for jenkinsapi requester (which is a wrapper around python-requests)
"""

import requests
import urllib.parse as urlparse

from jenkinsapi.custom_exceptions import JenkinsAPIException, PostRequired

# import logging

# these two lines enable debugging at httplib level
# (requests->urllib3->httplib)
# you will see the REQUEST, including HEADERS and DATA, and RESPONSE
# with HEADERS but without DATA.
# the only thing missing will be the response.body which is not logged.
# import httplib
# httplib.HTTPConnection.debuglevel = 1

# you need to initialize logging, otherwise you will not see anything
# from requests
# logging.basicConfig()
# logging.getLogger().setLevel(logging.DEBUG)
# requests_log = logging.getLogger("requests.packages.urllib3")
# requests_log.setLevel(logging.DEBUG)
# requests_log.propagate = True

requests.adapters.DEFAULT_RETRIES = 5


class Requester(object):
    """
    A class which carries out HTTP requests. You can replace this
    class with one of your own implementation if you require some other
    way to access Jenkins.

    This default class can handle simple authentication only.
    """

    VALID_STATUS_CODES = [
        200,
    ]
    AUTH_COOKIE = None

    def __init__(self, *args, **kwargs):
        username = None
        password = None
        ssl_verify = True
        cert = None
        baseurl = None
        timeout = 10
        max_retries = 3

        if len(args) == 1:
            (username,) = args
        elif len(args) == 2:
            username, password = args
        elif len(args) == 3:
            username, password, ssl_verify = args
        elif len(args) == 4:
            username, password, ssl_verify, cert = args
        elif len(args) == 5:
            username, password, ssl_verify, cert, baseurl = args
        elif len(args) == 6:
            username, password, ssl_verify, cert, baseurl, timeout = args
        elif len(args) > 6:
            raise ValueError("To much positional arguments given!")

        baseurl = kwargs.get("baseurl", baseurl)
        self.base_scheme = (
            urlparse.urlsplit(baseurl).scheme if baseurl else None
        )
        self.username = kwargs.get("username", username)
        self.password = kwargs.get("password", password)
        if self.username:
            assert self.password, (
                "Please provide both username and password "
                "or don't provide them at all"
            )
        if self.password:
            assert self.username, (
                "Please provide both username and password "
                "or don't provide them at all"
            )
        self.ssl_verify = kwargs.get("ssl_verify", ssl_verify)
        self.cert = kwargs.get("cert", cert)
        self.timeout = kwargs.get("timeout", timeout)
        self.session = requests.Session()
        self.max_retries = kwargs.get("max_retries", max_retries)
        if self.max_retries is not None:
            retry_adapter = requests.adapters.HTTPAdapter(
                max_retries=self.max_retries
            )
            self.session.mount("http://", retry_adapter)
            self.session.mount("https://", retry_adapter)

    def get_request_dict(
        self, params=None, data=None, files=None, headers=None, **kwargs
    ):
        requestKwargs = kwargs
        if self.username:
            requestKwargs["auth"] = (self.username, self.password)

        if params:
            assert isinstance(params, dict), (
                f"Params must be a dict, got {repr(params)}"
            )
            requestKwargs["params"] = params

        if headers:
            assert isinstance(headers, dict), (
                f"headers must be a dict, got {repr(headers)}"
            )
            requestKwargs["headers"] = headers

        if self.AUTH_COOKIE:
            currentheaders = requestKwargs.get("headers", {})
            currentheaders.update({"Cookie": self.AUTH_COOKIE})
            requestKwargs["headers"] = currentheaders

        requestKwargs["verify"] = self.ssl_verify
        requestKwargs["cert"] = self.cert

        if data:
            # It may seem odd, but some Jenkins operations require posting
            # an empty string.
            requestKwargs["data"] = data

        if files:
            requestKwargs["files"] = files

        requestKwargs["timeout"] = self.timeout

        return requestKwargs

    def _update_url_scheme(self, url):
        """
        Updates scheme of given url to the one used in Jenkins baseurl.
        """
        if self.base_scheme and not url.startswith("%s://" % self.base_scheme):
            url_split = urlparse.urlsplit(url)
            url = urlparse.urlunsplit(
                [
                    self.base_scheme,
                    url_split.netloc,
                    url_split.path,
                    url_split.query,
                    url_split.fragment,
                ]
            )
        return url

    def get_url(
        self,
        url,
        params=None,
        headers=None,
        allow_redirects=True,
        stream=False,
    ):
        requestKwargs = self.get_request_dict(
            params=params,
            headers=headers,
            allow_redirects=allow_redirects,
            stream=stream,
        )
        return self.session.get(self._update_url_scheme(url), **requestKwargs)

    def post_url(
        self,
        url,
        params=None,
        data=None,
        files=None,
        headers=None,
        allow_redirects=True,
        **kwargs,
    ):
        requestKwargs = self.get_request_dict(
            params=params,
            data=data,
            files=files,
            headers=headers,
            allow_redirects=allow_redirects,
            **kwargs,
        )
        return self.session.post(self._update_url_scheme(url), **requestKwargs)

    def post_xml_and_confirm_status(
        self, url, params=None, data=None, valid=None
    ):
        headers = {"Content-Type": "text/xml"}
        return self.post_and_confirm_status(
            url, params=params, data=data, headers=headers, valid=valid
        )

    def post_and_confirm_status(
        self,
        url,
        params=None,
        data=None,
        files=None,
        headers=None,
        valid=None,
        allow_redirects=True,
    ):
        valid = valid or self.VALID_STATUS_CODES
        if not headers and not files:
            headers = {"Content-Type": "application/x-www-form-urlencoded"}

        assert data is not None, "Post messages must have data"

        response = self.post_url(
            url, params, data, files, headers, allow_redirects
        )
        if response.status_code not in valid:
            raise JenkinsAPIException(
                "Operation failed. url={0}, data={1}, headers={2}, "
                "status={3}, text={4}".format(
                    response.url,
                    data,
                    headers,
                    response.status_code,
                    response.text.encode("UTF-8"),
                )
            )
        return response

    def get_and_confirm_status(
        self, url, params=None, headers=None, valid=None, stream=False
    ):
        valid = valid or self.VALID_STATUS_CODES
        response = self.get_url(url, params, headers, stream=stream)
        if response.status_code not in valid:
            if response.status_code == 405:  # POST required
                raise PostRequired("POST required for url {0}".format(url))

            raise JenkinsAPIException(
                "Operation failed. url={0}, headers={1}, status={2}, "
                "text={3}".format(
                    response.url,
                    headers,
                    response.status_code,
                    response.text.encode("UTF-8"),
                )
            )
        return response



"""
Get information about currently installed plugins
"""

from jenkinsapi.jenkins import Jenkins


plugin_name = "subversion"
jenkins = Jenkins("http://localhost:8080")
plugin = jenkins.get_plugins()[plugin_name]

print(repr(plugin))



"""
System tests for `jenkinsapi.jenkins` module.
"""

import os
from posixpath import join
import re
import time
import gzip
import shutil
import tempfile
import logging
from jenkinsapi_tests.systests.job_configs import JOB_WITH_ARTIFACTS
from jenkinsapi_tests.test_utils.random_strings import random_string

log = logging.getLogger(__name__)


def test_artifacts(jenkins):
    job_name = "create_%s" % random_string()
    job = jenkins.create_job(job_name, JOB_WITH_ARTIFACTS)
    job.invoke(block=True)

    build = job.get_last_build()

    while build.is_running():
        time.sleep(1)

    artifacts = build.get_artifact_dict()
    assert isinstance(artifacts, dict) is True

    text_artifact = artifacts["out.txt"]
    binary_artifact = artifacts["out.gz"]

    tempDir = tempfile.mkdtemp()

    try:
        # Verify that we can handle text artifacts
        text_artifact.save_to_dir(tempDir, strict_validation=True)
        text_file_path = join(tempDir, text_artifact.filename)
        assert os.path.exists(text_file_path)
        with open(text_file_path, "rb") as f:
            read_back_text = f.read().strip()
            read_back_text = read_back_text.decode("ascii")
            log.info("Text artifact: %s", read_back_text)
            assert (
                re.match(r"^PING \S+ \(127.0.0.1\)", read_back_text)
                is not None
            )
            assert read_back_text.endswith("ms") is True

        # Verify that we can hande binary artifacts
        binary_artifact.save_to_dir(tempDir, strict_validation=True)
        bin_file_path = join(tempDir, binary_artifact.filename)
        assert os.path.exists(bin_file_path)
        with gzip.open(bin_file_path, "rb") as f:
            read_back_text = f.read().strip()
            read_back_text = read_back_text.decode("ascii")
            assert (
                re.match(r"^PING \S+ \(127.0.0.1\)", read_back_text)
                is not None
            )
            assert read_back_text.endswith("ms") is True
    finally:
        shutil.rmtree(tempDir)



"""
This module implements the Executors class, which is intended to be a
container-like interface for all of the executors defined on a single
Jenkins node.
"""

from __future__ import annotations

import logging
from typing import Iterator

from jenkinsapi.executor import Executor
from jenkinsapi.jenkinsbase import JenkinsBase

log: logging.Logger = logging.getLogger(__name__)


class Executors(JenkinsBase):
    """
    This class provides a container-like API which gives
    access to all executors on a Jenkins node.

    Returns a list of Executor Objects.
    """

    def __init__(
        self, baseurl: str, nodename: str, jenkins: "Jenkins"
    ) -> None:
        self.nodename: str = nodename
        self.jenkins: str = jenkins
        JenkinsBase.__init__(self, baseurl)
        self.count: int = self._data["numExecutors"]

    def __str__(self) -> str:
        return f"Executors @ {self.baseurl}"

    def get_jenkins_obj(self) -> "Jenkins":
        return self.jenkins

    def __iter__(self) -> Iterator[Executor]:
        for index in range(self.count):
            executor_url = "%s/executors/%s" % (self.baseurl, index)
            yield Executor(executor_url, self.nodename, self.jenkins, index)



"""
Module for JenkinsBase class
"""

from __future__ import annotations

import ast
import pprint
import logging
from urllib.parse import quote
from jenkinsapi import config
from jenkinsapi.custom_exceptions import JenkinsAPIException

logger = logging.getLogger(__name__)


class JenkinsBase(object):
    """
    This appears to be the base object that all other jenkins objects are
    inherited from
    """

    def __repr__(self):
        return """<%s.%s %s>""" % (
            self.__class__.__module__,
            self.__class__.__name__,
            str(self),
        )

    def __str__(self):
        raise NotImplementedError

    def __init__(self, baseurl: str, poll: bool = True):
        """
        Initialize a jenkins connection
        """
        self._data = None
        self.baseurl = self.strip_trailing_slash(baseurl)
        if poll:
            self.poll()

    def get_jenkins_obj(self):
        raise NotImplementedError(
            "Please implement this method on %s" % self.__class__.__name__
        )

    def __eq__(self, other) -> bool:
        """
        Return true if the other object represents a connection to the
        same server
        """
        if not isinstance(other, self.__class__):
            return False
        return other.baseurl == self.baseurl

    @classmethod
    def strip_trailing_slash(cls, url: str) -> str:
        while url.endswith("/"):
            url = url[:-1]
        return url

    def poll(self, tree=None):
        data = self._poll(tree=tree)
        if "jobs" in data:
            data["jobs"] = self.resolve_job_folders(data["jobs"])
        if not tree:
            self._data = data

        return data

    def _poll(self, tree=None):
        url = self.python_api_url(self.baseurl)
        return self.get_data(url, tree=tree)

    def get_data(self, url, params=None, tree=None):
        requester = self.get_jenkins_obj().requester
        if tree:
            if not params:
                params = {"tree": tree}
            else:
                params.update({"tree": tree})

        response = requester.get_url(url, params)
        if response.status_code != 200:
            logger.error(
                "Failed request at %s with params: %s %s",
                url,
                params,
                tree if tree else "",
            )
            response.raise_for_status()
        try:
            return ast.literal_eval(response.text)
        except Exception:
            logger.exception("Inappropriate content found at %s", url)
            raise JenkinsAPIException("Cannot parse %s" % response.content)

    def pprint(self):
        """
        Print out all the data in this object for debugging.
        """
        pprint.pprint(self._data)

    def resolve_job_folders(self, jobs):
        for job in list(jobs):
            if "color" not in job.keys():
                jobs.remove(job)
                jobs += self.process_job_folder(job, self.baseurl)

        return jobs

    def process_job_folder(self, folder, folder_path):
        logger.debug("Processing folder %s in %s", folder["name"], folder_path)
        folder_path += "/job/%s" % quote(folder["name"])
        data = self.get_data(
            self.python_api_url(folder_path), tree="jobs[name,color]"
        )

        result = []
        for job in data.get("jobs", []):
            if "color" not in job.keys():
                result += self.process_job_folder(job, folder_path)
            else:
                job["url"] = "%s/job/%s" % (folder_path, quote(job["name"]))
                result.append(job)

        return result

    @classmethod
    def python_api_url(cls, url: str) -> str:
        if url.endswith(config.JENKINS_API):
            return url
        else:
            if url.endswith(r"/"):
                fmt = "%s%s"
            else:
                fmt = "%s/%s"
            return fmt % (url, config.JENKINS_API)



"""
jenkinsapi plugins
"""

from __future__ import annotations

from typing import Generator
import logging
import time
import re
from io import BytesIO
from urllib.parse import urlencode
import json

import requests
from jenkinsapi.plugin import Plugin
from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.custom_exceptions import UnknownPlugin
from jenkinsapi.custom_exceptions import JenkinsAPIException
from jenkinsapi.utils.jsonp_to_json import jsonp_to_json
from jenkinsapi.utils.manifest import Manifest, read_manifest


log: logging.Logger = logging.getLogger(__name__)


class Plugins(JenkinsBase):
    """
    Plugins class for jenkinsapi
    """

    def __init__(self, url: str, jenkins_obj: "Jenkins") -> None:
        self.jenkins_obj: "Jenkins" = jenkins_obj
        JenkinsBase.__init__(self, url)

    def get_jenkins_obj(self) -> "Jenkins":
        return self.jenkins_obj

    def check_updates_server(self) -> None:
        url: str = (
            f"{self.jenkins_obj.baseurl}/pluginManager/checkUpdatesServer"
        )
        self.jenkins_obj.requester.post_and_confirm_status(
            url, params={}, data={}
        )

    @property
    def update_center_dict(self):
        update_center = "https://updates.jenkins.io/update-center.json"
        jsonp = requests.get(update_center).content.decode("utf-8")
        return json.loads(jsonp_to_json(jsonp))

    def _poll(self, tree=None):
        return self.get_data(self.baseurl, tree=tree)

    def keys(self) -> list[str]:
        return self.get_plugins_dict().keys()

    __iter__ = keys

    def iteritems(self) -> Generator[str, "Plugin"]:
        return self._get_plugins()

    def values(self) -> list["Plugin"]:
        return [a[1] for a in self.iteritems()]

    def _get_plugins(self) -> Generator[str, "Plugin"]:
        if "plugins" in self._data:
            for p_dict in self._data["plugins"]:
                yield p_dict["shortName"], Plugin(p_dict)

    def get_plugins_dict(self) -> dict[str, "Plugin"]:
        return dict(self._get_plugins())

    def __len__(self) -> int:
        return len(self.get_plugins_dict().keys())

    def __getitem__(self, plugin_name: str) -> Plugin:
        try:
            return self.get_plugins_dict()[plugin_name]
        except KeyError:
            raise UnknownPlugin(plugin_name)

    def __setitem__(self, shortName, plugin: "Plugin") -> None:
        """
        Installs plugin in Jenkins.

        If plugin already exists - this method is going to uninstall the
        existing plugin and install the specified version if it is not
        already installed.

        :param shortName: Plugin ID
        :param plugin a Plugin object to be installed.
        """
        if self.plugin_version_already_installed(plugin):
            return
        if plugin.is_latest(self.update_center_dict):
            self._install_plugin_from_updatecenter(plugin)
        else:
            self._install_specific_version(plugin)
        self._wait_until_plugin_installed(plugin)

    def _install_plugin_from_updatecenter(self, plugin: "Plugin") -> None:
        """
        Latest versions of plugins can be installed from the update
        center (and don't need a restart.)
        """
        xml_str: str = plugin.get_attributes()
        url: str = (
            "%s/pluginManager/installNecessaryPlugins"
            % self.jenkins_obj.baseurl
        )
        self.jenkins_obj.requester.post_xml_and_confirm_status(
            url, data=xml_str
        )

    @property
    def update_center_install_status(self):
        """
        Jenkins 2.x specific
        """
        url: str = "%s/updateCenter/installStatus" % self.jenkins_obj.baseurl
        status = self.jenkins_obj.requester.get_url(url)
        if status.status_code == 404:
            raise JenkinsAPIException(
                "update_center_install_status not available for Jenkins 1.X"
            )
        return status.json()

    @property
    def restart_required(self):
        """
        Call after plugin installation to check if Jenkins requires a restart
        """
        try:
            jobs = self.update_center_install_status["data"]["jobs"]
        except JenkinsAPIException:
            return True  # Jenkins 1.X has no update_center
        return any([job for job in jobs if job["requiresRestart"] == "true"])

    def _install_specific_version(self, plugin: "Plugin") -> None:
        """
        Plugins that are not the latest version have to be uploaded.
        """
        download_link: str = plugin.get_download_link(
            update_center_dict=self.update_center_dict
        )
        downloaded_plugin: BytesIO = self._download_plugin(download_link)
        plugin_dependencies = self._get_plugin_dependencies(downloaded_plugin)
        log.debug("Installing dependencies for plugin '%s'", plugin.shortName)
        self.jenkins_obj.install_plugins(plugin_dependencies)
        url = "%s/pluginManager/uploadPlugin" % self.jenkins_obj.baseurl
        requester = self.jenkins_obj.requester
        downloaded_plugin.seek(0)
        requester.post_and_confirm_status(
            url,
            files={"file": ("plugin.hpi", downloaded_plugin)},
            data={},
            params={},
        )

    def _get_plugin_dependencies(
        self, downloaded_plugin: BytesIO
    ) -> list["Plugin"]:
        """
        Returns a list of all dependencies for a downloaded plugin
        """
        plugin_dependencies = []
        manifest: Manifest = read_manifest(downloaded_plugin)
        manifest_dependencies = manifest.main_section.get(
            "Plugin-Dependencies"
        )
        if manifest_dependencies:
            dependencies = manifest_dependencies.split(",")
            for dep in dependencies:
                # split plugin:version;resolution:optional entries
                components = dep.split(";")
                dep_plugin = components[0]
                name = dep_plugin.split(":")[0]
                # install latest dependency, avoids multiple
                # versions of the same dep
                plugin_dependencies.append(
                    Plugin({"shortName": name, "version": "latest"})
                )
        return plugin_dependencies

    def _download_plugin(self, download_link):
        downloaded_plugin = BytesIO()
        downloaded_plugin.write(requests.get(download_link).content)
        return downloaded_plugin

    def _plugin_has_finished_installation(self, plugin) -> bool:
        """
        Return True if installation is marked as 'Success' or
        'SuccessButRequiresRestart' in Jenkins' update_center,
        else return False.
        """
        try:
            jobs = self.update_center_install_status["data"]["jobs"]
            for job in jobs:
                if job["name"] == plugin.shortName and job[
                    "installStatus"
                ] in [
                    "Success",
                    "SuccessButRequiresRestart",
                ]:
                    return True
            return False
        except JenkinsAPIException:
            return False  # lack of update_center in Jenkins 1.X

    def plugin_version_is_being_installed(self, plugin) -> bool:
        """
        Return true if plugin is currently being installed.
        """
        try:
            jobs = self.update_center_install_status["data"]["jobs"]
        except JenkinsAPIException:
            return False  # lack of update_center in Jenkins 1.X
        return any(
            [
                job
                for job in jobs
                if job["name"] == plugin.shortName
                and job["version"] == plugin.version
            ]
        )

    def plugin_version_already_installed(self, plugin) -> bool:
        """
        Check if plugin version is already installed
        """
        if plugin.shortName not in self:
            if self.plugin_version_is_being_installed(plugin):
                return True
            return False
        installed_plugin = self[plugin.shortName]
        if plugin.version == installed_plugin.version:
            return True
        elif plugin.version == "latest":
            # we don't have an exact version, we first check if Jenkins
            # knows about an update
            if (
                hasattr(installed_plugin, "hasUpdates")
                and installed_plugin.hasUpdates
            ):
                return False

            # Jenkins may not have an up-to-date catalogue,
            # so check update-center directly
            latest_version = self.update_center_dict["plugins"][
                plugin.shortName
            ]["version"]
            return installed_plugin.version == latest_version

        return False

    def __delitem__(self, shortName):
        if re.match(".*@.*", shortName):
            real_shortName = re.compile("(.*)@(.*)").search(shortName).group(1)
            raise ValueError(
                ("Plugin shortName can't contain version. '%s' should be '%s'")
                % (shortName, real_shortName)
            )
        if shortName not in self:
            raise KeyError(
                'Plugin with ID "%s" not found, cannot uninstall' % shortName
            )
        if self[shortName].deleted:
            raise JenkinsAPIException(
                'Plugin "%s" already marked for uninstall. '
                "Restart jenkins for uninstall to complete."
            )
        params = {"Submit": "OK", "json": {}}
        url = "%s/pluginManager/plugin/%s/doUninstall" % (
            self.jenkins_obj.baseurl,
            shortName,
        )
        self.jenkins_obj.requester.post_and_confirm_status(
            url, params={}, data=urlencode(params)
        )

        self.poll()
        if not self[shortName].deleted:
            raise JenkinsAPIException(
                "Problem uninstalling plugin '%s'." % shortName
            )

    def _wait_until_plugin_installed(self, plugin, maxwait=120, interval=1):
        for _ in range(maxwait, 0, -interval):
            self.poll()
            if self._plugin_has_finished_installation(plugin):
                return True
            if plugin.shortName in self:
                return True  # for Jenkins 1.X
            time.sleep(interval)

        if self.jenkins_obj.version.startswith("2"):
            raise JenkinsAPIException(
                "Problem installing plugin '%s'." % plugin.shortName
            )

        log.warning(
            "Plugin '%s' not found in loaded plugins."
            "You may need to restart Jenkins.",
            plugin.shortName,
        )
        return False

    def __contains__(self, plugin_name):
        """
        True if plugin_name is the name of a defined plugin
        """
        return plugin_name in self.keys()

    def __str__(self):
        plugins = [
            plugin["shortName"] for plugin in self._data.get("plugins", [])
        ]
        return str(sorted(plugins))



"""
About this library
==================

Jenkins is the market leading continuous integration system, originally created
by Kohsuke Kawaguchi. This API makes Jenkins even easier to use by providing an
easy to use conventional python interface.

Jenkins (and It's predecessor Hudson) are fantastic projects - but they are
somewhat Java-centric. Thankfully the designers have provided an excellent and
complete REST interface. This library wraps up that interface as more
conventional python objects in order to make most Jenkins oriented
tasks simpler.

This library can help you:

 * Query the test-results of a completed build
 * Get a objects representing the latest builds of a job
 * Search for artifacts by simple criteria
 * Block until jobs are complete
 * Install artifacts to custom-specified directory structures
 * username/password auth support for jenkins instances with auth turned on
 * Ability to search for builds by subversion revision
 * Ability to add/remove/query jenkins slaves

Installing JenkinsAPI
=====================

pip install jenkinsapi

Project Authors
===============

 * Salim Fadhley (sal@stodge.org)
 * Ramon van Alteren (ramon@vanalteren.nl)
 * Ruslan Lutsenko (ruslan.lutcenko@gmail.com)
 * Aleksey Maksimov
 * Clinton Steiner

Current code lives on github: https://github.com/pycontribs/jenkinsapi

"""

from importlib.metadata import version

from jenkinsapi import (
    # Modules
    command_line,
    utils,
    # Files
    api,
    artifact,
    build,
    config,
    constants,
    custom_exceptions,
    fingerprint,
    executors,
    executor,
    jenkins,
    jenkinsbase,
    job,
    node,
    result_set,
    result,
    view,
)

__all__ = [
    "command_line",
    "utils",
    "api",
    "artifact",
    "build",
    "config",
    "constants",
    "custom_exceptions",
    "executors",
    "executor",
    "fingerprint",
    "jenkins",
    "jenkinsbase",
    "job",
    "node",
    "result_set",
    "result",
    "view",
]
__docformat__ = "epytext"
__version__ = version("jenkinsapi")



# flake8: noqa
#    import mock
#
#    # To run unittests on python 2.6 please use unittest2 library
#    try:
#        import unittest2 as unittest
#    except ImportError:
#        import unittest
#
#    from jenkinsapi import config
#    from jenkinsapi.job import Job
#    from jenkinsapi.jenkinsbase import JenkinsBase
#
#
#    CFG_NODE = """
#    <project>
#    <scm class="hudson.plugins.mercurial.MercurialSCM" plugin="mercurial@1.42">
#    <source>http://cm5/hg/sandbox/v01.0/int</source>
#    <modules/>
#    <clean>false</clean>
#    <browser class="hudson.plugins.mercurial.browser.HgWeb">
#    <url>http://cm5/hg/sandbox/v01.0/int/</url>
#    </browser>
#    </scm>
#    </project>
#    """
#
#    # TODO: Make JOB_DATA to be one coming from Hg job
#    class TestHgJob(unittest.TestCase):
#        JOB_DATA = {
#            "actions": [],
#            "description": "test job",
#            "displayName": "foo",
#            "displayNameOrNull": None,
#            "name": "foo",
#            "url": "http://halob:8080/job/foo/",
#            "buildable": True,
#            "builds": [
#                {"number": 3, "url": "http://halob:8080/job/foo/3/"},
#                {"number": 2, "url": "http://halob:8080/job/foo/2/"},
#                {"number": 1, "url": "http://halob:8080/job/foo/1/"},
#            ],
#            "color": "blue",
#            "firstBuild": {"number": 1, "url": "http://halob:8080/job/foo/1/"},
#            "healthReport": [
#                {
#                    "description": "Build stability: No recent builds failed.",
#                    "iconUrl": "health-80plus.png",
#                    "score": 100,
#                }
#            ],
#            "inQueue": False,
#            "keepDependencies": False,
#            # build running
#            "lastBuild": {"number": 4, "url": "http://halob:8080/job/foo/4/"},
#            "lastCompletedBuild": {
#                "number": 3,
#                "url": "http://halob:8080/job/foo/3/",
#            },
#            "lastFailedBuild": None,
#            "lastStableBuild": {
#                "number": 3,
#                "url": "http://halob:8080/job/foo/3/",
#            },
#            "lastSuccessfulBuild": {
#                "number": 3,
#                "url": "http://halob:8080/job/foo/3/",
#            },
#            "lastUnstableBuild": None,
#            "lastUnsuccessfulBuild": None,
#            "nextBuildNumber": 4,
#            "property": [],
#            "queueItem": None,
#            "concurrentBuild": False,
#            "downstreamProjects": [],
#            "scm": {},
#            "upstreamProjects": [],
#        }
#
#        URL_DATA = {"http://halob:8080/job/foo/%s" % config.JENKINS_API: JOB_DATA}
#
#        def fakeGetData(self, url, *args, **kwargs):
#            try:
#                return TestHgJob.URL_DATA[url]
#            except KeyError:
#                raise Exception("Missing data for %s" % url)
#
#        @mock.patch.object(JenkinsBase, "get_data", fakeGetData)
#        def setUp(self):
#            self.J = mock.MagicMock()  # Jenkins object
#            self.j = Job("http://halob:8080/job/foo/", "foo", self.J)
#
#        def configtree_with_branch(self):
#            config_node = CFG_NODE
#            return config_node
#
#        def configtree_with_default_branch(self):
#            config_node = CFG_NODE
#            return config_node
#
#        def configtree_multibranch_git(self):
#            config_node = """
#    <flow-definition plugin="workflow-job@2.35">
#        <keepDependencies>false</keepDependencies>
#        <properties>
#            <org.jenkinsci.plugins.workflow.job.properties.PipelineTriggersJobProperty>
#                <triggers>
#                    <hudson.triggers.TimerTrigger>
#                        <spec>H H * * H(6-7)</spec>
#                    </hudson.triggers.TimerTrigger>
#                    <jenkins.triggers.ReverseBuildTrigger>
#                        <spec></spec>
#                        <upstreamProjects></upstreamProjects>
#                        <threshold>
#                            <name>SUCCESS</name>
#                            <ordinal>0</ordinal>
#                            <color>BLUE</color>
#                            <completeBuild>true</completeBuild>
#                        </threshold>
#                    </jenkins.triggers.ReverseBuildTrigger>
#                </triggers>
#            </org.jenkinsci.plugins.workflow.job.properties.PipelineTriggersJobProperty>
#            <jenkins.model.BuildDiscarderProperty>
#                <strategy class="hudson.tasks.LogRotator">
#                    <daysToKeep>-1</daysToKeep>
#                    <numToKeep>5</numToKeep>
#                    <artifactDaysToKeep>-1</artifactDaysToKeep>
#                    <artifactNumToKeep>5</artifactNumToKeep>
#                </strategy>
#            </jenkins.model.BuildDiscarderProperty>
#            <org.jenkinsci.plugins.workflow.multibranch.BranchJobProperty
#                    plugin="workflow-multibranch@2.21">
#                <branch plugin="branch-api@2.5.4">
#                    <sourceId>a2d4bcda-6141-4af2-8088-39139a147902</sourceId>
#                    <head class="com.cloudbees.jenkins.plugins.bitbucket.BranchSCMHead"
#                            plugin="cloudbees-bitbucket-branch-source@2.5.0">
#                        <name>master</name>
#                        <repositoryType>GIT</repositoryType>
#                    </head>
#                    <scm class="hudson.plugins.git.GitSCM" plugin="git@3.12.1">
#                        <configVersion>2</configVersion>
#                        <userRemoteConfigs>
#                            <hudson.plugins.git.UserRemoteConfig>
#                                <name>origin</name>
#                                <refspec>+refs/heads/master:refs/remotes/origin/master</refspec>
#                                <url>ssh://git@bitbucket.site/project-name/reponame.git</url>
#                                <credentialsId>jenkins-stash</credentialsId>
#                            </hudson.plugins.git.UserRemoteConfig>
#                        </userRemoteConfigs>
#                        <branches class="singleton-list">
#                            <hudson.plugins.git.BranchSpec>
#                                <name>master</name>
#                            </hudson.plugins.git.BranchSpec>
#                        </branches>
#                        <doGenerateSubmoduleConfigurations>false</doGenerateSubmoduleConfigurations>
#                        <browser class="hudson.plugins.git.browser.BitbucketWeb">
#                            <url>https://bitbucket.site/projects/project-name/repos/reponame</url>
#                        </browser>
#                        <submoduleCfg class="empty-list"/>
#                        <extensions>
#                            <jenkins.plugins.git.GitSCMSourceDefaults>
#                                <includeTags>false</includeTags>
#                            </jenkins.plugins.git.GitSCMSourceDefaults>
#                        </extensions>
#                    </scm>
#                    <properties/>
#                </branch>
#            </org.jenkinsci.plugins.workflow.multibranch.BranchJobProperty>
#        </properties>
#        <definition class="org.jenkinsci.plugins.workflow.multibranch.SCMBinder"
#                plugin="workflow-multibranch@2.21">
#            <scriptPath>Jenkinsfile</scriptPath>
#        </definition>
#        <triggers/>
#        <disabled>false</disabled>
#    </flow-definition>
#            """
#            return config_node
#
#        @mock.patch.object(Job, "get_config", configtree_with_branch)
#        def test_hg_attributes(self):
#            expected_url = ["http://cm5/hg/sandbox/v01.0/int"]
#            self.j.load_config()
#            self.assertEqual(self.j.get_scm_type(), "hg")
#            self.assertEqual(self.j.get_scm_url(), expected_url)
#            self.assertEqual(self.j.get_scm_branch(), ["testme"])
#
#        @mock.patch.object(Job, "get_config", configtree_with_default_branch)
#        def test_hg_attributes_default_branch(self):
#            self.j.load_config()
#            self.assertEqual(self.j.get_scm_branch(), ["default"])
#
#        @mock.patch.object(Job, "get_config", configtree_multibranch_git)
#        def test_git_attributes_multibranch(self):
#            expected_url = ["ssh://git@bitbucket.site/project-name/reponame.git"]
#            self.j.load_config()
#            self.assertEqual(self.j.get_scm_type(), "git")
#            self.assertEqual(self.j.get_scm_url(), expected_url)
#            self.assertEqual(self.j.get_scm_branch(), ["master"])
#
#
#    if __name__ == "__main__":
#        unittest.main()



"""
Save this file as server.py
>>> python server.py 0.0.0.0 8001
serving on 0.0.0.0:8001

or simply

>>> python server.py
Serving on localhost:8000

You can use this to test GET and POST methods.
"""

import http.server as SimpleHTTPServer
import socketserver
import logging
import cgi

PORT = 8081  # <-- change this to be the actual port you want to run on
INTERFACE = "localhost"


class ServerHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
    def do_GET(self):
        logging.warning("======= GET STARTED =======")
        logging.warning(self.headers)
        SimpleHTTPServer.SimpleHTTPRequestHandler.do_GET(self)

    def do_POST(self):
        logging.warning("======= POST STARTED =======")
        logging.warning(self.headers)
        form = cgi.FieldStorage(
            fp=self.rfile,
            headers=self.headers,
            environ={
                "REQUEST_METHOD": "POST",
                "CONTENT_TYPE": self.headers["Content-Type"],
            },
        )
        logging.warning("======= POST VALUES =======")
        for item in form.list:
            logging.warning(item)
        logging.warning("\n")
        SimpleHTTPServer.SimpleHTTPRequestHandler.do_GET(self)


Handler = ServerHandler

httpd = socketserver.TCPServer(("", PORT), Handler)

print(
    "Serving at: http://%(interface)s:%(port)s"
    % dict(interface=INTERFACE or "localhost", port=PORT)
)
httpd.serve_forever()



"""
Queue module for jenkinsapi
"""

from __future__ import annotations

from typing import Iterator, Tuple
import logging
import time
from requests import HTTPError
from jenkinsapi.jenkinsbase import JenkinsBase
from jenkinsapi.custom_exceptions import UnknownQueueItem, NotBuiltYet

log: logging.Logger = logging.getLogger(__name__)


class Queue(JenkinsBase):
    """
    Class that represents the Jenkins queue
    """

    def __init__(self, baseurl: str, jenkins_obj: "Jenkins") -> None:
        """
        Init the Jenkins queue object
        :param baseurl: basic url for the queue
        :param jenkins_obj: ref to the jenkins obj
        """
        self.jenkins: "Jenkins" = jenkins_obj
        JenkinsBase.__init__(self, baseurl)

    def __str__(self) -> str:
        return self.baseurl

    def get_jenkins_obj(self) -> "Jenkins":
        return self.jenkins

    def iteritems(self) -> Iterator[Tuple[str, "QueueItem"]]:
        for item in self._data["items"]:
            queue_id = item["id"]
            item_baseurl = "%s/item/%i" % (self.baseurl, queue_id)
            yield (
                item["id"],
                QueueItem(baseurl=item_baseurl, jenkins_obj=self.jenkins),
            )

    def iterkeys(self) -> Iterator[str]:
        for item in self._data["items"]:
            yield item["id"]

    def itervalues(self) -> Iterator["QueueItem"]:
        for item in self._data["items"]:
            yield QueueItem(self.jenkins, **item)

    def keys(self) -> list[str]:
        return list(self.iterkeys())

    def values(self) -> list["QueueItem"]:
        return list(self.itervalues())

    def __len__(self) -> int:
        return len(self._data["items"])

    def __getitem__(self, item_id: str) -> "QueueItem":
        self_as_dict = dict(self.iteritems())
        if item_id in self_as_dict:
            return self_as_dict[item_id]
        else:
            raise UnknownQueueItem(item_id)

    def _get_queue_items_for_job(self, job_name: str) -> Iterator["QueueItem"]:
        for item in self._data["items"]:
            if "name" in item["task"] and item["task"]["name"] == job_name:
                yield QueueItem(
                    self.get_queue_item_url(item), jenkins_obj=self.jenkins
                )

    def get_queue_items_for_job(self, job_name: str):
        return list(self._get_queue_items_for_job(job_name))

    def get_queue_item_url(self, item: str) -> str:
        return "%s/item/%i" % (self.baseurl, item["id"])

    def delete_item(self, queue_item: "QueueItem"):
        self.delete_item_by_id(queue_item.queue_id)

    def delete_item_by_id(self, item_id: str):
        deleteurl: str = "%s/cancelItem?id=%s" % (self.baseurl, item_id)
        self.get_jenkins_obj().requester.post_url(deleteurl)


class QueueItem(JenkinsBase):
    """An individual item in the queue"""

    def __init__(self, baseurl: str, jenkins_obj: "Jenkins") -> None:
        self.jenkins: "Jenkins" = jenkins_obj
        JenkinsBase.__init__(self, baseurl)

    @property
    def queue_id(self):
        return self._data["id"]

    @property
    def name(self):
        return self._data["task"]["name"]

    @property
    def why(self):
        return self._data.get("why")

    def get_jenkins_obj(self) -> "Jenkins":
        return self.jenkins

    def get_job(self) -> "Job":
        """
        Return the job associated with this queue item
        """
        return self.jenkins.get_job_by_url(
            self._data["task"]["url"],
            self._data["task"]["name"],
        )

    def get_parameters(self):
        """returns parameters of queue item"""
        actions = self._data.get("actions", [])
        for action in actions:
            if isinstance(action, dict) and "parameters" in action:
                parameters = action["parameters"]
                return dict(
                    [(x["name"], x.get("value", None)) for x in parameters]
                )
        return []

    def __repr__(self) -> str:
        return "<%s.%s %s>" % (
            self.__class__.__module__,
            self.__class__.__name__,
            str(self),
        )

    def __str__(self) -> str:
        return "%s Queue #%i" % (self.name, self.queue_id)

    def get_build(self) -> "Build":
        build_number = self.get_build_number()
        job = self.get_job()
        return job[build_number]

    def block_until_complete(self, delay=5):
        build = self.block_until_building(delay)
        return build.block_until_complete(delay=delay)

    def block_until_building(self, delay=5):
        while True:
            try:
                self.poll()
                return self.get_build()
            except NotBuiltYet:
                time.sleep(delay)
                continue
            except HTTPError as http_error:
                log.debug(str(http_error))
                time.sleep(delay)
                continue

    def is_running(self) -> bool:
        """Return True if this queued item is running."""
        try:
            return self.get_build().is_running()
        except NotBuiltYet:
            return False

    def is_queued(self) -> bool:
        """Return True if this queued item is queued."""
        try:
            self.get_build()
        except NotBuiltYet:
            return True
        else:
            return False

    def get_build_number(self) -> int:
        try:
            return self._data["executable"]["number"]
        except (KeyError, TypeError):
            raise NotBuiltYet()

    def get_job_name(self) -> str:
        try:
            return self._data["task"]["name"]
        except KeyError:
            raise NotBuiltYet()



"""
Search for job artifacts
"""

from jenkinsapi.api import search_artifacts

jenkinsurl = "http://localhost:8080"
jobid = "foo"
# I need a build that contains all of these
artifact_ids = ["test1.txt", "test2.txt"]
result = search_artifacts(jenkinsurl, jobid, artifact_ids)
print((repr(result)))



"""
System tests for generation new api token for logged in user
"""

import pytest
import logging
from jenkinsapi.utils.crumb_requester import CrumbRequester


log = logging.getLogger(__name__)


@pytest.mark.generate_new_api_token
def test_generate_new_api_token(jenkins_admin_admin):
    jenkins_admin_admin.requester = CrumbRequester(
        baseurl=jenkins_admin_admin.baseurl,
        username=jenkins_admin_admin.username,
        password=jenkins_admin_admin.password,
    )
    jenkins_admin_admin.poll()

    new_token = (
        jenkins_admin_admin.generate_new_api_token()
    )  # generate new token
    log.info("newly generated token: %s", new_token)
    assert new_token is not None



import pytest
import mock
from jenkinsapi.jenkins import JenkinsBase


@pytest.fixture(scope="function")
def jenkinsbase():
    return JenkinsBase("http://localhost:8080/", poll=False)


def test_called_in__poll(jenkinsbase, monkeypatch, mocker):
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return {
            "description": "My jobs",
            "jobs": [
                {
                    "name": "Foo",
                    "url": "http://localhost:8080/job/Foo",
                    "color": "blue",
                }
            ],
            "name": "All",
            "property": [],
            "url": "http://localhost:8080/view/All/",
        }

    monkeypatch.setattr(JenkinsBase, "_poll", fake_poll)
    stub = mocker.stub()
    monkeypatch.setattr(JenkinsBase, "resolve_job_folders", stub)

    jenkinsbase.poll()

    stub.assert_called_once_with(
        [
            {
                "name": "Foo",
                "url": "http://localhost:8080/job/Foo",
                "color": "blue",
            },
        ],
    )


def test_no_folders(jenkinsbase):
    jobs = [
        {
            "name": "Foo",
            "url": "http://localhost:8080/job/Foo",
            "color": "blue",
        },
        {
            "name": "Bar",
            "url": "http://localhost:8080/job/Bar",
            "color": "disabled",
        },
    ]

    assert jenkinsbase.resolve_job_folders(jobs) == [
        {
            "name": "Foo",
            "url": "http://localhost:8080/job/Foo",
            "color": "blue",
        },
        {
            "name": "Bar",
            "url": "http://localhost:8080/job/Bar",
            "color": "disabled",
        },
    ]


def test_empty_folder(jenkinsbase, monkeypatch, mocker):
    def fake_get_data(cls, url, tree=None):  # pylint: disable=unused-argument
        return {"jobs": []}

    monkeypatch.setattr(JenkinsBase, "get_data", fake_get_data)
    spy = mocker.spy(jenkinsbase, "get_data")

    jobs = [
        {
            "name": "Folder1",
            "url": "http://localhost:8080/job/Folder1",
        },
    ]

    assert jenkinsbase.resolve_job_folders(jobs) == []
    spy.assert_called_once_with(
        "http://localhost:8080/job/Folder1/api/python", tree="jobs[name,color]"
    )


def test_folder_job_mix(jenkinsbase, monkeypatch, mocker):
    def fake_get_data(cls, url, tree=None):  # pylint: disable=unused-argument
        return {
            "jobs": [
                {
                    "name": "Bar",
                    "url": "http://localhost:8080/job/Folder1/job/Bar",
                    "color": "disabled",
                }
            ]
        }

    monkeypatch.setattr(JenkinsBase, "get_data", fake_get_data)
    spy = mocker.spy(jenkinsbase, "get_data")

    jobs = [
        {
            "name": "Foo",
            "url": "http://localhost:8080/job/Foo",
            "color": "blue",
        },
        {
            "name": "Folder1",
            "url": "http://localhost:8080/job/Folder1",
        },
    ]

    assert jenkinsbase.resolve_job_folders(jobs) == [
        {
            "name": "Foo",
            "url": "http://localhost:8080/job/Foo",
            "color": "blue",
        },
        {
            "name": "Bar",
            "url": "http://localhost:8080/job/Folder1/job/Bar",
            "color": "disabled",
        },
    ]

    spy.assert_called_once_with(
        "http://localhost:8080/job/Folder1/api/python", tree="jobs[name,color]"
    )


def test_multiple_folders(jenkinsbase, monkeypatch, mocker):
    def fake_get_data(cls, url, tree=None):  # pylint: disable=unused-argument
        # first call
        if "Folder1" in url:
            return {
                "jobs": [
                    {
                        "name": "Foo",
                        "url": "http://localhost:8080/job/Folder1/job/Foo",
                        "color": "disabled",
                    },
                ]
            }

        if "Folder2" in url:
            # second call
            return {
                "jobs": [
                    {
                        "name": "Bar",
                        "url": "http://localhost:8080/job/Folder2/job/Bar",
                        "color": "blue",
                    },
                ]
            }

    monkeypatch.setattr(JenkinsBase, "get_data", fake_get_data)
    spy = mocker.spy(jenkinsbase, "get_data")

    jobs = [
        {
            "name": "Folder1",
            "url": "http://localhost:8080/job/Folder1",
        },
        {
            "name": "Folder2",
            "url": "http://localhost:8080/job/Folder2",
        },
    ]

    assert jenkinsbase.resolve_job_folders(jobs) == [
        {
            "name": "Foo",
            "url": "http://localhost:8080/job/Folder1/job/Foo",
            "color": "disabled",
        },
        {
            "name": "Bar",
            "url": "http://localhost:8080/job/Folder2/job/Bar",
            "color": "blue",
        },
    ]

    assert spy.call_args_list == [
        mock.call(
            "http://localhost:8080/job/Folder1/api/python",
            tree="jobs[name,color]",
        ),
        mock.call(
            "http://localhost:8080/job/Folder2/api/python",
            tree="jobs[name,color]",
        ),
    ]


def test_multiple_folder_levels(jenkinsbase, monkeypatch, mocker):
    def fake_get_data(cls, url, tree=None):  # pylint: disable=unused-argument
        if "Folder1" in url and "Folder2" not in url:
            # first call
            return {
                "jobs": [
                    {
                        "name": "Bar",
                        "url": "http://localhost:8080/job/Folder1/job/Bar",
                        "color": "disabled",
                    },
                    {
                        "name": "Folder2",
                        "url": "http://localhost:8080/job/Folder1/job/Folder2",
                    },
                ]
            }

        if "Folder2" in url:
            # second call
            return {
                "jobs": [
                    {
                        "name": "Baz",
                        "url": (
                            "http://localhost:8080/job/Folder1/"
                            "job/Folder2/job/Baz"
                        ),
                        "color": "disabled",
                    },
                ]
            }

    monkeypatch.setattr(JenkinsBase, "get_data", fake_get_data)
    spy = mocker.spy(jenkinsbase, "get_data")

    jobs = [
        {
            "name": "Foo",
            "url": "http://localhost:8080/job/Foo",
            "color": "blue",
        },
        {
            "name": "Folder1",
            "url": "http://localhost:8080/job/Folder1",
        },
    ]

    assert jenkinsbase.resolve_job_folders(jobs) == [
        {
            "name": "Foo",
            "url": "http://localhost:8080/job/Foo",
            "color": "blue",
        },
        {
            "name": "Bar",
            "url": "http://localhost:8080/job/Folder1/job/Bar",
            "color": "disabled",
        },
        {
            "name": "Baz",
            "url": ("http://localhost:8080/job/Folder1/job/Folder2/job/Baz"),
            "color": "disabled",
        },
    ]

    assert spy.call_args_list == [
        mock.call(
            "http://localhost:8080/job/Folder1/api/python",
            tree="jobs[name,color]",
        ),
        mock.call(
            "http://localhost:8080/job/Folder1/job/Folder2/api/python",
            tree="jobs[name,color]",
        ),
    ]



import pytest
import mock
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.executors import Executors
from jenkinsapi.executor import Executor


DATAM = {
    "assignedLabels": [{}],
    "description": None,
    "jobs": [],
    "mode": "NORMAL",
    "nodeDescription": "the master Jenkins node",
    "nodeName": "",
    "numExecutors": 2,
    "overallLoad": {},
    "primaryView": {"name": "All", "url": "http://localhost:8080/"},
    "quietingDown": False,
    "slaveAgentPort": 0,
    "unlabeledLoad": {},
    "useCrumbs": False,
    "useSecurity": False,
    "views": [
        {"name": "All", "url": "http://localhost:8080/"},
        {"name": "BigMoney", "url": "http://localhost:8080/view/BigMoney/"},
    ],
}

DATA0 = {
    "actions": [],
    "displayName": "host0.host.com",
    "executors": [{}, {}, {}, {}, {}, {}, {}, {}],
    "icon": "computer.png",
    "idle": False,
    "jnlpAgent": True,
    "launchSupported": False,
    "loadStatistics": {},
    "manualLaunchAllowed": True,
    "monitorData": {
        "hudson.node_monitors.SwapSpaceMonitor": {
            "availablePhysicalMemory": 8462417920,
            "availableSwapSpace": 0,
            "totalPhysicalMemory": 75858042880,
            "totalSwapSpace": 0,
        },
        "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
        "hudson.node_monitors.ResponseTimeMonitor": {"average": 2},
        "hudson.node_monitors.TemporarySpaceMonitor": {
            "path": "/tmp",
            "size": 430744551424,
        },
        "hudson.node_monitors.DiskSpaceMonitor": {
            "path": "/data/jenkins",
            "size": 1214028627968,
        },
        "hudson.node_monitors.ClockMonitor": {"diff": 1},
    },
    "numExecutors": 8,
    "offline": False,
    "offlineCause": None,
    "offlineCauseReason": "",
    "oneOffExecutors": [{}, {}],
    "temporarilyOffline": False,
}

DATA1 = {
    "actions": [],
    "displayName": "host1.host.com",
    "executors": [{}, {}],
    "icon": "computer.png",
    "idle": False,
    "jnlpAgent": True,
    "launchSupported": False,
    "loadStatistics": {},
    "manualLaunchAllowed": True,
    "monitorData": {
        "hudson.node_monitors.SwapSpaceMonitor": {
            "availablePhysicalMemory": 8462417920,
            "availableSwapSpace": 0,
            "totalPhysicalMemory": 75858042880,
            "totalSwapSpace": 0,
        },
        "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
        "hudson.node_monitors.ResponseTimeMonitor": {"average": 2},
        "hudson.node_monitors.TemporarySpaceMonitor": {
            "path": "/tmp",
            "size": 430744551424,
        },
        "hudson.node_monitors.DiskSpaceMonitor": {
            "path": "/data/jenkins",
            "size": 1214028627968,
        },
        "hudson.node_monitors.ClockMonitor": {"diff": 1},
    },
    "numExecutors": 2,
    "offline": False,
    "offlineCause": None,
    "offlineCauseReason": "",
    "oneOffExecutors": [{}, {}],
    "temporarilyOffline": False,
}

DATA2 = {
    "actions": [],
    "displayName": "host2.host.com",
    "executors": [{}, {}, {}, {}],
    "icon": "computer.png",
    "idle": False,
    "jnlpAgent": True,
    "launchSupported": False,
    "loadStatistics": {},
    "manualLaunchAllowed": True,
    "monitorData": {
        "hudson.node_monitors.SwapSpaceMonitor": {
            "availablePhysicalMemory": 8462417920,
            "availableSwapSpace": 0,
            "totalPhysicalMemory": 75858042880,
            "totalSwapSpace": 0,
        },
        "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
        "hudson.node_monitors.ResponseTimeMonitor": {"average": 2},
        "hudson.node_monitors.TemporarySpaceMonitor": {
            "path": "/tmp",
            "size": 430744551424,
        },
        "hudson.node_monitors.DiskSpaceMonitor": {
            "path": "/data/jenkins",
            "size": 1214028627968,
        },
        "hudson.node_monitors.ClockMonitor": {"diff": 1},
    },
    "numExecutors": 4,
    "offline": False,
    "offlineCause": None,
    "offlineCauseReason": "",
    "oneOffExecutors": [{}, {}],
    "temporarilyOffline": False,
}

DATA3 = {
    "actions": [],
    "displayName": "host3.host.com",
    "executors": [{}],
    "icon": "computer.png",
    "idle": False,
    "jnlpAgent": True,
    "launchSupported": False,
    "loadStatistics": {},
    "manualLaunchAllowed": True,
    "monitorData": {
        "hudson.node_monitors.SwapSpaceMonitor": {
            "availablePhysicalMemory": 8462417920,
            "availableSwapSpace": 0,
            "totalPhysicalMemory": 75858042880,
            "totalSwapSpace": 0,
        },
        "hudson.node_monitors.ArchitectureMonitor": "Linux (amd64)",
        "hudson.node_monitors.ResponseTimeMonitor": {"average": 2},
        "hudson.node_monitors.TemporarySpaceMonitor": {
            "path": "/tmp",
            "size": 430744551424,
        },
        "hudson.node_monitors.DiskSpaceMonitor": {
            "path": "/data/jenkins",
            "size": 1214028627968,
        },
        "hudson.node_monitors.ClockMonitor": {"diff": 1},
    },
    "numExecutors": 1,
    "offline": False,
    "offlineCause": None,
    "offlineCauseReason": "",
    "oneOffExecutors": [{}, {}],
    "temporarilyOffline": False,
}

EXEC0 = {
    "currentExecutable": {
        "number": 4168,
        "url": "http://localhost:8080/job/testjob/4168/",
    },
    "currentWorkUnit": {},
    "idle": False,
    "likelyStuck": False,
    "number": 0,
    "progress": 48,
}

EXEC1 = {
    "currentExecutable": None,
    "currentWorkUnit": None,
    "idle": True,
    "likelyStuck": False,
    "number": 0,
    "progress": -1,
}


@pytest.fixture(scope="function")
def jenkins(monkeypatch):
    def fake_poll(cls, tree=None):  # pylint: disable=unused-argument
        return DATAM

    monkeypatch.setattr(Jenkins, "_poll", fake_poll)
    return Jenkins("http://localhost:8080")


def test_repr(jenkins):
    # Can we produce a repr string for this object
    assert repr(jenkins)


def test_check_url(jenkins):
    assert jenkins.baseurl == "http://localhost:8080"


def test_get_executors(jenkins, monkeypatch):
    def fake_poll_extr(cls, tree=None):  # pylint: disable=unused-argument
        return EXEC0

    def fake_poll_extrs(cls, tree=None):  # pylint: disable=unused-argument
        return DATA3

    monkeypatch.setattr(Executor, "_poll", fake_poll_extr)
    monkeypatch.setattr(Executors, "_poll", fake_poll_extrs)

    exec_info = jenkins.get_executors(DATA3["displayName"])

    assert isinstance(exec_info, object)
    assert isinstance(repr(exec_info), str)

    for ex in exec_info:
        assert ex.get_progress() == 48, "Should return 48 %"


def testis_idle(jenkins, monkeypatch):
    def fake_poll_extr(cls, tree=None):  # pylint: disable=unused-argument
        return EXEC1

    def fake_poll_extrs(cls, tree=None):  # pylint: disable=unused-argument
        return DATA3

    monkeypatch.setattr(Executor, "_poll", fake_poll_extr)
    monkeypatch.setattr(Executors, "_poll", fake_poll_extrs)

    exec_info = jenkins.get_executors("host3.host.com")

    assert isinstance(exec_info, object)
    for ex in exec_info:
        assert ex.get_progress() == -1, "Should return 48 %"
        assert ex.is_idle() is True, "Should return True"
        assert repr(ex) == "<jenkinsapi.executor.Executor host3.host.com 0>"


@mock.patch.object(Executor, "_poll")
def test_likely_stuck(jenkins, monkeypatch):
    def fake_poll_extr(cls, tree=None):  # pylint: disable=unused-argument
        return EXEC0

    monkeypatch.setattr(Executor, "_poll", fake_poll_extr)

    baseurl = "http://localhost:8080/computer/host0.host.com/executors/0"
    nodename = "host0.host.com"
    single_executer = Executor(baseurl, nodename, jenkins, "0")
    assert single_executer.likely_stuck() is False


def test_get_current_executable(jenkins, monkeypatch):
    def fake_poll_extr(cls, tree=None):  # pylint: disable=unused-argument
        return EXEC0

    monkeypatch.setattr(Executor, "_poll", fake_poll_extr)

    baseurl = "http://localhost:8080/computer/host0.host.com/executors/0"
    nodename = "host0.host.com"
    single_executer = Executor(baseurl, nodename, jenkins, "0")
    assert single_executer.get_current_executable()["number"] == 4168
    assert (
        single_executer.get_current_executable()["url"]
        == "http://localhost:8080/job/testjob/4168/"
    )



"""
How to create slaves/nodes
"""

import logging
import requests
from jenkinsapi.jenkins import Jenkins
from jenkinsapi.utils.requester import Requester

requests.packages.urllib3.disable_warnings()

log_level = getattr(logging, "DEBUG")
logging.basicConfig(level=log_level)
logger = logging.getLogger()

jenkins_url = "http://localhost:8080/"
username = "default_user"  # In case Jenkins requires authentication
password = "default_password"

jenkins = Jenkins(
    jenkins_url,
    requester=Requester(
        username, password, baseurl=jenkins_url, ssl_verify=False
    ),
)

# Create JNLP(Java Webstart) slave
node_dict = {
    "num_executors": 1,  # Number of executors
    "node_description": "Test JNLP Node",  # Just a user friendly text
    "remote_fs": "/tmp",  # Remote workspace location
    "labels": "my_new_node",  # Space separated labels string
    "exclusive": True,  # Only run jobs assigned to it
}
new_jnlp_node = jenkins.nodes.create_node("My new webstart node", node_dict)

node_dict = {
    "num_executors": 1,
    "node_description": "Test SSH Node",
    "remote_fs": "/tmp",
    "labels": "new_node",
    "exclusive": True,
    "host": "localhost",  # Remote hostname
    "port": 22,  # Remote post, usually 22
    "credential_description": "localhost cred",  # Credential to use
    # [Mandatory for SSH node!]
    # (see Credentials example)
    "jvm_options": "-Xmx2000M",  # JVM parameters
    "java_path": "/bin/java",  # Path to java
    "prefix_start_slave_cmd": "",
    "suffix_start_slave_cmd": "",
    "max_num_retries": 0,
    "retry_wait_time": 0,
    "retention": "OnDemand",  # Change to 'Always' for
    # immediate slave launch
    "ondemand_delay": 1,
    "ondemand_idle_delay": 5,
    "env": [  # Environment variables
        {"key": "TEST", "value": "VALUE"},
        {"key": "TEST2", "value": "value2"},
    ],
}
new_ssh_node = jenkins.nodes.create_node("My new SSH node", node_dict)

# Take this slave offline
if new_ssh_node.is_online():
    new_ssh_node.toggle_temporarily_offline()

    # Take this slave back online
    new_ssh_node.toggle_temporarily_offline()

# Get a list of all slave names
slave_names = jenkins.nodes.keys()

# Get Node object
my_node = jenkins.nodes["My new SSH node"]
# Take this slave offline
my_node.set_offline()

# Delete slaves
del jenkins.nodes["My new webstart node"]
del jenkins.nodes["My new SSH node"]



"""
An example of how to use JenkinsAPI to fetch the config XML of a job.
"""

from jenkinsapi.jenkins import Jenkins

jenkins = Jenkins("http://localhost:8080")
jobName = jenkins.keys()[0]  # get the first job

config = jenkins[jobName].get_config()

print(config)



"""
Search for job artifacts using regexp
"""

import re
from jenkinsapi.api import search_artifact_by_regexp

jenkinsurl = "http://localhost:8080"
jobid = "foo"
artifact_regexp = re.compile(r"test1\.txt")  # A file name I want.
result = search_artifact_by_regexp(jenkinsurl, jobid, artifact_regexp)
print((repr(result)))



import mock

# To run unittests on python 2.6 please use unittest2 library
try:
    import unittest2 as unittest
except ImportError:
    import unittest

from jenkinsapi.result_set import ResultSet
from jenkinsapi.result import Result


class TestResultSet(unittest.TestCase):
    DATA = {
        "duration": 0.0,
        "failCount": 2,
        "passCount": 0,
        "skipCount": 0,
        "suites": [
            {
                "cases": [
                    {
                        "age": 1,
                        "className": "<nose.suite.ContextSuite context=jenkinsapi_tests",  # noqa
                        "duration": 0.0,
                        "errorDetails": "Timeout error occured while waiting for Jenkins start.",  # noqa
                        "errorStackTrace": 'Traceback (most recent call last):\n  File "/usr/lib/python2.7/dist-packages/nose/suite.py", line 208, in run\n    self.setUp()\n  File "/usr/lib/python2.7/dist-packages/nose/suite.py", line 291, in setUp\n    self.setupContext(ancestor)\n  File "/usr/lib/python2.7/dist-packages/nose/suite.py", line 314, in setupContext\n    try_run(context, names)\n  File "/usr/lib/python2.7/dist-packages/nose/util.py", line 478, in try_run\n    return func()\n  File "/var/lib/jenkins/jobs/test_jenkinsapi/workspace/jenkinsapi/src/jenkinsapi_tests/systests/__init__.py", line 54, in setUpPackage\n    launcher = JenkinsLauncher(update_war=True, launch=True)\n  File "/var/lib/jenkins/jobs/test_jenkinsapi/workspace/jenkinsapi/src/jenkinsapi_tests/systests/__init__.py", line 20, in __init__\n    self.launch()\n  File "/var/lib/jenkins/jobs/test_jenkinsapi/workspace/jenkinsapi/src/jenkinsapi_tests/systests/__init__.py", line 41, in launch\n    raise Timeout(\'Timeout error occured while waiting for Jenkins start.\')\nTimeout: Timeout error occured while waiting for Jenkins start.\n',  # noqa
                        "failedSince": 88,
                        "name": "systests>:setup",
                        "skipped": False,
                        "status": "FAILED",
                        "stderr": None,
                        "stdout": None,
                    },
                    {
                        "age": 1,
                        "className": "nose.failure.Failure",
                        "duration": 0.0,
                        "errorDetails": "No module named mock",
                        "errorStackTrace": 'Traceback (most recent call last):\n  File "/usr/lib/python2.7/unittest/case.py", line 332, in run\n    testMethod()\n  File "/usr/lib/python2.7/dist-packages/nose/loader.py", line 390, in loadTestsFromName\n    addr.filename, addr.module)\n  File "/usr/lib/python2.7/dist-packages/nose/importer.py", line 39, in importFromPath\n    return self.importFromDir(dir_path, fqname)\n  File "/usr/lib/python2.7/dist-packages/nose/importer.py", line 86, in importFromDir\n    mod = load_module(part_fqname, fh, filename, desc)\n  File "/var/lib/jenkins/jobs/test_jenkinsapi/workspace/jenkinsapi/src/jenkinsapi_tests/unittests/test_build.py", line 1, in <module>\n    import mock\nImportError: No module named mock\n',  # noqa
                        "failedSince": 88,
                        "name": "runTest",
                        "skipped": False,
                        "status": "FAILED",
                        "stderr": None,
                        "stdout": None,
                    },
                ],
                "duration": 0.0,
                "id": None,
                "name": "nosetests",
                "stderr": None,
                "stdout": None,
                "timestamp": None,
            }
        ],
        "childReports": [
            {"child": {"number": 1915, "url": "url1"}, "result": None},
        ],
    }

    @mock.patch.object(ResultSet, "_poll")
    def setUp(self, _poll):
        _poll.return_value = self.DATA

        # def __init__(self, url, build ):

        self.b = mock.MagicMock()  # Build object
        self.b.__str__.return_value = "FooBuild"
        self.rs = ResultSet("http://", self.b)

    def testRepr(self):
        # Can we produce a repr string for this object
        repr(self.rs)

    def testName(self):
        with self.assertRaises(AttributeError):
            self.rs.id()

        self.assertEqual(self.rs.name, "Test Result for FooBuild")

    def testBuildComponents(self):
        self.assertTrue(self.rs.items())
        for k, v in self.rs.items():
            self.assertIsInstance(k, str)
            self.assertIsInstance(v, Result)
            self.assertIsInstance(v.identifier(), str)


if __name__ == "__main__":
    unittest.main()



import pytest
import requests
from jenkinsapi.jenkins import Requester
from jenkinsapi.custom_exceptions import JenkinsAPIException
from mock import patch


def test_no_parameters_uses_default_values():
    req = Requester()
    assert isinstance(req, Requester)
    assert req.username is None
    assert req.password is None
    assert req.ssl_verify
    assert req.cert is None
    assert req.base_scheme is None
    assert req.timeout == 10


def test_all_named_parameters():
    req = Requester(
        username="foo",
        password="bar",
        ssl_verify=False,
        cert="foobar",
        baseurl="http://dummy",
        timeout=5,
    )
    assert isinstance(req, Requester)
    assert req.username == "foo"
    assert req.password == "bar"
    assert not req.ssl_verify
    assert req.cert == "foobar"
    assert req.base_scheme == "http", "dummy"
    assert req.timeout == 5


def test_mix_one_unnamed_named_parameters():
    req = Requester(
        "foo",
        password="bar",
        ssl_verify=False,
        cert="foobar",
        baseurl="http://dummy",
        timeout=5,
    )
    assert isinstance(req, Requester)
    assert req.username == "foo"
    assert req.password == "bar"
    assert not req.ssl_verify
    assert req.cert == "foobar"
    assert req.base_scheme == "http", "dummy"
    assert req.timeout == 5


def test_mix_two_unnamed_named_parameters():
    req = Requester(
        "foo",
        "bar",
        ssl_verify=False,
        cert="foobar",
        baseurl="http://dummy",
        timeout=5,
    )
    assert isinstance(req, Requester)
    assert req.username == "foo"
    assert req.password == "bar"
    assert not req.ssl_verify
    assert req.cert == "foobar"
    assert req.base_scheme == "http", "dummy"
    assert req.timeout == 5


def test_mix_three_unnamed_named_parameters():
    req = Requester(
        "foo", "bar", False, cert="foobar", baseurl="http://dummy", timeout=5
    )
    assert isinstance(req, Requester)
    assert req.username == "foo"
    assert req.password == "bar"
    assert not req.ssl_verify
    assert req.cert == "foobar"
    assert req.base_scheme == "http", "dummy"
    assert req.timeout == 5


def test_mix_four_unnamed_named_parameters():
    req = Requester(
        "foo", "bar", False, "foobar", baseurl="http://dummy", timeout=5
    )
    assert isinstance(req, Requester)
    assert req.username == "foo"
    assert req.password == "bar"
    assert not req.ssl_verify
    assert req.cert == "foobar"
    assert req.base_scheme == "http", "dummy"
    assert req.timeout == 5


def test_mix_five_unnamed_named_parameters():
    req = Requester("foo", "bar", False, "foobar", "http://dummy", timeout=5)
    assert isinstance(req, Requester)
    assert req.username == "foo"
    assert req.password == "bar"
    assert not req.ssl_verify
    assert req.cert == "foobar"
    assert req.base_scheme == "http", "dummy"
    assert req.timeout == 5


def test_all_unnamed_parameters():
    req = Requester("foo", "bar", False, "foobar", "http://dummy", 5)
    assert isinstance(req, Requester)
    assert req.username == "foo"
    assert req.password == "bar"
    assert not req.ssl_verify
    assert req.cert == "foobar"
    assert req.base_scheme == "http", "dummy"
    assert req.timeout == 5


def test_to_much_unnamed_parameters_raises_error():
    with pytest.raises(Exception):
        Requester("foo", "bar", False, "foobar", "http://dummy", 5, "test")


def test_username_without_password_raises_error():
    with pytest.raises(Exception):
        Requester(username="foo")
        Requester("foo")


def test_password_without_username_raises_error():
    with pytest.raises(AssertionError):
        Requester(password="bar")


def test_get_request_dict_auth():
    req = Requester("foo", "bar")

    req_return = req.get_request_dict(params={}, data=None, headers=None)
    assert isinstance(req_return, dict)
    assert req_return.get("auth")
    assert req_return["auth"] == ("foo", "bar")


@patch("jenkinsapi.jenkins.Requester.AUTH_COOKIE", "FAKE")
def test_get_request_dict_cookie():
    req = Requester("foo", "bar")

    req_return = req.get_request_dict(params={}, data=None, headers=None)
    assert isinstance(req_return, dict)
    assert req_return.get("headers")
    assert req_return.get("headers").get("Cookie")
    assert req_return.get("headers").get("Cookie") == "FAKE"


@patch("jenkinsapi.jenkins.Requester.AUTH_COOKIE", "FAKE")
def test_get_request_dict_updatecookie():
    req = Requester("foo", "bar")

    req_return = req.get_request_dict(
        params={}, data=None, headers={"key": "value"}
    )
    assert isinstance(req_return, dict)
    assert req_return.get("headers")
    assert req_return.get("headers").get("key")
    assert req_return.get("headers").get("key") == "value"
    assert req_return.get("headers").get("Cookie")
    assert req_return.get("headers").get("Cookie") == "FAKE"


def test_get_request_dict_nocookie():
    req = Requester("foo", "bar")

    req_return = req.get_request_dict(params={}, data=None, headers=None)
    assert isinstance(req_return, dict)
    assert not req_return.get("headers")


def test_get_request_dict_wrong_params():
    req = Requester("foo", "bar")

    with pytest.raises(AssertionError) as na:
        req.get_request_dict(params="wrong", data=None, headers=None)
    assert "Params must be a dict, got 'wrong'" in str(na.value)


def test_get_request_dict_correct_params():
    req = Requester("foo", "bar")

    req_return = req.get_request_dict(
        params={"param": "value"}, data=None, headers=None
    )

    assert isinstance(req_return, dict)
    assert req_return.get("params")
    assert req_return["params"] == {"param": "value"}


def test_get_request_dict_wrong_headers():
    req = Requester("foo", "bar")

    with pytest.raises(AssertionError) as na:
        req.get_request_dict(params={}, data=None, headers="wrong")
    assert "headers must be a dict, got 'wrong'" in str(na.value)


def test_get_request_dict_correct_headers():
    req = Requester("foo", "bar")

    req_return = req.get_request_dict(
        params={"param": "value"}, data=None, headers={"header": "value"}
    )

    assert isinstance(req_return, dict)
    assert req_return.get("headers")
    assert req_return["headers"] == {"header": "value"}


def test_get_request_dict_data_passed():
    req = Requester("foo", "bar")

    req_return = req.get_request_dict(
        params={"param": "value"},
        data="some data",
        headers={"header": "value"},
    )

    assert isinstance(req_return, dict)
    assert req_return.get("data")
    assert req_return["data"] == "some data"


def test_get_request_dict_data_not_passed():
    req = Requester("foo", "bar")

    req_return = req.get_request_dict(
        params={"param": "value"}, data=None, headers={"header": "value"}
    )

    assert isinstance(req_return, dict)
    assert req_return.get("data") is None


def test_get_url_get(monkeypatch):
    def fake_get(*args, **kwargs):  # pylint: disable=unused-argument
        return "SUCCESS"

    monkeypatch.setattr(requests.Session, "get", fake_get)

    req = Requester("foo", "bar")
    response = req.get_url(
        "http://dummy", params={"param": "value"}, headers=None
    )

    assert response == "SUCCESS"


def test_get_url_post(monkeypatch):
    def fake_post(*args, **kwargs):  # pylint: disable=unused-argument
        return "SUCCESS"

    monkeypatch.setattr(requests.Session, "post", fake_post)

    req = Requester("foo", "bar")
    response = req.post_url(
        "http://dummy", params={"param": "value"}, headers=None
    )

    assert response == "SUCCESS"


def test_post_xml_empty_xml(monkeypatch):
    def fake_post(*args, **kwargs):  # pylint: disable=unused-argument
        return "SUCCESS"

    monkeypatch.setattr(requests.Session, "post", fake_post)

    req = Requester("foo", "bar")
    with pytest.raises(AssertionError):
        req.post_xml_and_confirm_status(
            url="http://dummy", params={"param": "value"}, data=None
        )


def test_post_xml_and_confirm_status_some_xml(monkeypatch):
    class FakeResponse(requests.Response):
        def __init__(self, *args, **kwargs):  # pylint: disable=unused-argument
            self.status_code = 200

    def fake_post(*args, **kwargs):  # pylint: disable=unused-argument
        return FakeResponse()

    monkeypatch.setattr(requests.Session, "post", fake_post)

    req = Requester("foo", "bar")
    ret = req.post_xml_and_confirm_status(
        url="http://dummy", params={"param": "value"}, data="<xml/>"
    )
    assert isinstance(ret, requests.Response)


def test_post_and_confirm_status_empty_data(monkeypatch):
    def fake_post(*args, **kwargs):  # pylint: disable=unused-argument
        return "SUCCESS"

    monkeypatch.setattr(requests.Session, "post", fake_post)

    req = Requester("foo", "bar")
    with pytest.raises(AssertionError):
        req.post_and_confirm_status(
            url="http://dummy", params={"param": "value"}, data=None
        )


def test_post_and_confirm_status_some_data(monkeypatch):
    class FakeResponse(requests.Response):
        def __init__(self, *args, **kwargs):  # pylint: disable=unused-argument
            self.status_code = 200

    def fake_post(*args, **kwargs):  # pylint: disable=unused-argument
        return FakeResponse()

    monkeypatch.setattr(requests.Session, "post", fake_post)

    req = Requester("foo", "bar")
    ret = req.post_and_confirm_status(
        url="http://dummy", params={"param": "value"}, data="some data"
    )
    assert isinstance(ret, requests.Response)


def test_post_and_confirm_status_bad_result(monkeypatch):
    class FakeResponse(object):
        def __init__(self, *args, **kwargs):  # pylint: disable=unused-argument
            self.status_code = 500
            self.url = "http://dummy"
            self.text = "something"

    def fake_post(*args, **kwargs):  # pylint: disable=unused-argument
        return FakeResponse()

    monkeypatch.setattr(requests.Session, "post", fake_post)

    req = Requester("foo", "bar")
    with pytest.raises(JenkinsAPIException) as error:
        req.post_and_confirm_status(
            url="http://dummy", params={"param": "value"}, data="some data"
        )
    assert "status=500" in str(error)


def test_get_and_confirm_status(monkeypatch):
    class FakeResponse(requests.Response):
        def __init__(self, *args, **kwargs):  # pylint: disable=unused-argument
            self.status_code = 200

    def fake_get(*args, **kwargs):  # pylint: disable=unused-argument
        return FakeResponse()

    monkeypatch.setattr(requests.Session, "get", fake_get)

    req = Requester("foo", "bar")
    ret = req.get_and_confirm_status(
        url="http://dummy", params={"param": "value"}
    )
    assert isinstance(ret, requests.Response)


def test_get_and_confirm_status_bad_result(monkeypatch):
    class FakeResponse(object):
        def __init__(self, *args, **kwargs):  # pylint: disable=unused-argument
            self.status_code = 500
            self.url = "http://dummy"
            self.text = "something"

    def fake_get(*args, **kwargs):  # pylint: disable=unused-argument
        return FakeResponse()

    monkeypatch.setattr(requests.Session, "get", fake_get)

    req = Requester("foo", "bar", baseurl="http://dummy")
    with pytest.raises(JenkinsAPIException) as error:
        req.get_and_confirm_status(
            url="http://dummy", params={"param": "value"}
        )
    assert "status=500" in str(error)


def test_configure_max_retries():
    req = Requester(
        "username", "password", baseurl="http://dummy", max_retries=3
    )
    for adapter in req.session.adapters.values():
        assert adapter.max_retries.total == 3
